@@ -16,8 +16,7 @@
 module Text.CSL.Eval
     ( evalLayout
     , evalSorting
-    , rtfParser
-    , last'
+    , last', split, trim
     , module Text.CSL.Eval.Common
     , module Text.CSL.Eval.Output
     ) where
@@ -26,6 +25,7 @@ import Control.Arrow
 import Control.Applicative ( (<$>) )
 import Control.Monad.State
 import Data.Char
+import qualified Data.Map as M
 import Data.Maybe
 
 import Text.CSL.Eval.Common
@@ -36,15 +36,14 @@ import Text.CSL.Output.Plain
 import Text.CSL.Reference
 import Text.CSL.Style
 import Text.Pandoc.Definition
-import Text.ParserCombinators.Parsec hiding ( State (..) )
 
 -- | Produce the output with a 'Layout', the 'EvalMode', a 'Bool'
 -- 'True' if the evaluation happens for disambiguation purposes, the
--- 'Locale', the 'MacrpMap', the position of the cite and the
+-- 'Locale', the 'MacroMap', the position of the cite and the
 -- 'Reference'.
 evalLayout :: Layout   -> EvalMode -> Bool -> [Locale] -> [MacroMap]
-           -> [Option] -> Reference -> [Output]
-evalLayout (Layout _ _ es) em b l m o r
+           -> [Option] -> [Abbrev] -> Reference -> [Output]
+evalLayout (Layout _ _ es) em b l m o a r
     = cleanOutput evalOut
     where
       evalOut = case evalState job initSt of
@@ -62,15 +61,16 @@ evalLayout (Layout _ _ es) em b l m o r
                  EvalSorting c -> c
                  EvalBiblio  s -> emptyCite { citePosition = s }
       initSt = EvalState (mkRefMap r) (Env cit (localeTermMap locale) m
-                         (localeDate locale) o []) [] em b False [] [] False [] [] []
+                         (localeDate locale) o [] a) [] em b False [] [] False [] [] []
 
-evalSorting :: EvalMode -> [Locale] -> [MacroMap] -> [Option] -> [Sort] -> Reference -> [Sorting]
-evalSorting m l ms opts ss r
+evalSorting :: EvalMode -> [Locale] -> [MacroMap] -> [Option] ->
+               [Sort] -> [Abbrev] -> Reference -> [Sorting]
+evalSorting m l ms opts ss as r
     = map (format . sorting) ss
     where
       render       = renderPlainStrict . formatOutputList
       format (s,e) = applaySort s . render $ uncurry eval e
-      eval     o e = evalLayout (Layout emptyFormatting [] [e]) m False l ms o r
+      eval     o e = evalLayout (Layout emptyFormatting [] [e]) m False l ms o as r
       applaySort c s
           | Ascending {} <- c = Ascending  s
           | otherwise         = Descending s
@@ -108,12 +108,18 @@ evalElement el
                                     ifEmpty (evalNames False s n d)
                                             (withNames s el $ evalElements sub)
                                             (appendOutput fm)
-    | Substitute (e:els)    <- el = ifEmpty (consuming $ evalElement e)
+    | Substitute (e:els)    <- el = ifEmpty (consuming $ substituteWith e)
                                             (getFirst els) id
     | ShortNames   s fm d   <- el = head <$> gets (names . env) >>= \(Names _ ns fm' d' _) ->
                                     appendOutput fm' <$> evalNames False s (updateNameParts d fm ns) d'
     | otherwise                   = return []
     where
+      substituteWith e = head <$> gets (names . env) >>= \(Names _ ns fm d _) -> do
+                           case e of
+                             Names rs [Name NotSet _ [] [] []] fm' d' []
+                                 -> evalElement $ Names rs ns (mergeFM fm' fm) (d' `betterThen` d) []
+                             _   -> evalElement e
+
       updateNameParts d fm (Name f fm' nf d' np : xs) = Name f (mergeFM fm' fm) nf (d `betterThen` d') np : xs
       updateNameParts d fm                   (x : xs) = x : updateNameParts d fm xs
       updateNameParts _ _                         []  = []
@@ -140,14 +146,16 @@ evalElement el
                                     (getFirst xs)
 
       getMacro         s = maybe [] id . lookup s <$> gets (macros . env)
-      getVariable f fm s = case s of
+      getVariable f fm s = case (map toLower s) of
                              "year-suffix" -> getStringVar "ref-id" >>= \k  ->
                                               return . return $ OYearSuf [] k [] fm
                              "page"        -> getStringVar "page" >>= formatRange fm
                              "title"       -> formatTitle f fm
                              "locator"     -> getLocVar >>= formatRange fm . snd
-                             _             -> gets (options . env) >>= \opts ->
-                                              getVar [] (getFormattedValue opts f fm) s >>= \r ->
+                             "url"         -> getStringVar "url" >>= \k ->
+                                              if null k then return [] else return [OUrl k fm]
+                             _             -> gets (env >>> options &&& abbrevs) >>= \(opts,as) ->
+                                              getVar [] (getFormattedValue opts as f fm s) s >>= \r ->
                                               consumeVariable s >> return r
 
 evalIfThen :: IfThen -> [IfThen] -> [Element] -> State EvalState [Output]
@@ -178,7 +186,6 @@ evalIfThen i ei e
 
       chkType         t = let chk = (==) (formatVariable t) . show . fromMaybe NoType . fromValue
                           in  getVar False chk "ref-type"
-      numericVars       = ["edition", "volume", "number-of-volumes", "number", "issue", "citation-number"]
       chkNumeric      v = do val <- getStringVar v
                              return (v `elem` numericVars && or (map isDigit val))
       chkDate         v = getDateVar v >>= return . not . null . filter ((/=) [] . circa)
@@ -195,9 +202,9 @@ evalIfThen i ei e
                                           b == "ibid-with-locator-c" then True  else False
           | otherwise                = isIbid b
 
-getFormattedValue :: [Option] -> Form -> Formatting -> Value -> [Output]
-getFormattedValue o f fm val
-    | Just v <- fromValue val :: Maybe String    = rtfParser fm v
+getFormattedValue :: [Option] -> [Abbrev] -> Form -> Formatting -> String -> Value -> [Output]
+getFormattedValue o as f fm s val
+    | Just v <- fromValue val :: Maybe String    = rtfParser fm . getAbbr $ value v
     | Just v <- fromValue val :: Maybe Int       = output  fm (if v == 0 then [] else show v)
     | Just v <- fromValue val :: Maybe CNum      = if v == 0 then [] else [OCitNum (unCNum v) fm]
     | Just v <- fromValue val :: Maybe [RefDate] = formatDate (EvalSorting emptyCite) [] [] sortDate v
@@ -205,83 +212,31 @@ getFormattedValue o f fm val
                                                               fm nameOpts []) v
     | otherwise                                  = []
     where
+      value     = if stripPeriods fm then filter (/= '.') else id
+      getAbbr v = if f /= Short then v else
+                  case lookup "default" as of
+                    Nothing -> v
+                    Just x  -> case lookup s x of
+                                 Nothing -> v
+                                 Just x' -> case M.lookup v x' of
+                                              Nothing  -> v
+                                              Just x'' -> x''
       nameOpts = ("name-as-sort-order","all") : o
       sortDate = [ DatePart "year"  "numeric-leading-zeros" "" emptyFormatting
                  , DatePart "month" "numeric-leading-zeros" "" emptyFormatting
                  , DatePart "day"   "numeric-leading-zeros" "" emptyFormatting]
 
-rtfTags :: [(String, (String,Formatting))]
-rtfTags =
-    [("b"                      , ("b"   , ef {fontWeight    = "bold"      }))
-    ,("i"                      , ("i"   , ef {fontStyle     = "italic"    }))
-    ,("sc"                     , ("sc"  , ef {fontVariant   = "small-caps"}))
-    ,("sup"                    , ("sup" , ef {verticalAlign = "sub"       }))
-    ,("sub"                    , ("sub" , ef {verticalAlign = "sub"       }))
-    ,("span class=\"nocase\""  , ("span", ef {noCase        = True        }))
-    ,("span class=\"nodecor\"" , ("span", ef {noDecor       = True        }))
-    ]
-    where
-      ef = emptyFormatting
-
-rtfParser :: Formatting -> String -> [Output]
-rtfParser _ [] = []
-rtfParser fm s
-    = either (const [OStr s fm]) (return . flip Output fm . concat) $
-      parse (manyTill parser eof) "" s
-    where
-      parser = parseText <|> parseMarkup
-
-      parseText = do
-        let amper = string "&" >> notFollowedBy (char '#') >>
-                    return [OStr "&" emptyFormatting]
-        x  <- many $ noneOf "<'\"`“‘&"
-        xs <- parseQuotes <|> parseMarkup <|> amper
-        r  <- manyTill anyChar eof
-        return (OStr x emptyFormatting : xs ++
-                [Output (rtfParser emptyFormatting r) emptyFormatting])
-
-      parseMarkup = do
-        let tillTag = many $ noneOf "<"
-        m   <- string "<" >> manyTill anyChar (try $ string ">")
-        res <- case lookup m rtfTags of
-                 Just tf -> do let ot = "<"  ++ fst tf ++ ">"
-                                   ct = "</" ++ fst tf ++ ">"
-                                   parseGreedy = do a <- tillTag
-                                                    _ <- string ct
-                                                    return a
-                               x <- manyTill anyChar $ try $ string ct
-                               y <- try parseGreedy <|> (string ot >> pzero) <|> return []
-                               let r = if null y then x else x ++ ct ++ y
-                               return [Output (rtfParser emptyFormatting r) (snd tf)]
-                 Nothing -> do r <- tillTag
-                               return [OStr ("<" ++ m ++ ">" ++ r) emptyFormatting]
-        return [Output res emptyFormatting]
-
-      parseQuotes = choice [parseQ "'" "'"
-                           ,parseQ "\"" "\""
-                           ,parseQ "``" "''"
-                           ,parseQ "`" "'"
-                           ,parseQ "“" "”"
-                           ,parseQ "‘" "’"
-                           ,parseQ "&#39;" "&#39;"
-                           ,parseQ "&#34;" "&#34;"
-                           ,parseQ "&quot;" "&quot;"
-                           ,parseQ "&apos;" "&apos;"
-                           ]
-      parseQ a b = try $ do
-        q <- string a >> manyTill anyChar (try $ string b)
-        return [Output (rtfParser emptyFormatting q) (emptyFormatting {quotes = True})]
-
 formatTitle :: Form -> Formatting -> State EvalState [Output]
 formatTitle f fm
-    | Short <- f = getIt "short-title" "title"
-    | otherwise  = getIt "title" "short-title"
+    | Short <- f = getIt "title-short" "title"
+    | otherwise  = getIt "title" "title-short"
     where
       getIt x fb = do
         o <- gets (options . env)
-        r <- getVar [] (getFormattedValue o f fm) x
+        a <- gets (abbrevs . env)
+        r <- getVar [] (getFormattedValue o a f fm x) x
         case r of
-          [] -> getVar [] (getFormattedValue o f fm) fb
+          [] -> getVar [] (getFormattedValue o a f fm x) fb
           _  -> return r
 
 formatNumber :: NumericForm -> Formatting -> String -> State EvalState [Output]
@@ -309,18 +264,18 @@ formatRange _ [] = return []
 formatRange fm p = do
   ops <- gets (options . env)
   let opt = getOptionVal "page-range-format" ops
-      splitRange = second (dropWhile (== '-')) . break (== '-')
+      splitRange = break (== '-') >>> trim *** trim . dropWhile (== '-')
       pages = map splitRange . split (== ',') $ p
       format a b = if b /= []
-                   then flip Output emptyFormatting [ OStr a emptyFormatting, OPan [EnDash]
+                   then flip Output emptyFormatting [ OStr a emptyFormatting, OPan [Str "\x2013"]
                                                     , OStr b emptyFormatting]
                    else flip Output emptyFormatting [ OStr a emptyFormatting]
       result = case opt of
                  "expanded" -> map (uncurry format . uncurry expandedRange) pages
                  "chicago"  -> map (uncurry format . uncurry chicagoRange ) pages
                  "minimal"  -> map (uncurry format . uncurry minimalRange ) pages
-                 _          -> [OStr p emptyFormatting]
-  return [flip Output fm $ addDelim ", " result]
+                 _          -> map (uncurry format                        ) pages
+  return [flip OLoc fm $ addDelim ", " result]
 
 expandedRange :: String -> String -> (String, String)
 expandedRange sa [] = (sa,[])
