@@ -49,6 +49,7 @@ data Environment
       , dates   :: [Element]
       , options :: [Option]
       , names   :: [Element]
+      , abbrevs :: [Abbrev]
       } deriving ( Show )
 
 data EvalMode
@@ -128,6 +129,7 @@ isVarSet s = if s /= "locator"
 withRefMap :: (ReferenceMap -> a) -> State EvalState a
 withRefMap f = return . f =<< gets ref
 
+-- | Convert variable to lower case, translating underscores ("_") to dashes ("-")
 formatVariable :: String -> String
 formatVariable = foldr f []
     where f x xs = if x == '_' then '-' : xs else toLower x : xs
