@@ -26,7 +26,7 @@ import Text.CSL.Eval.Output
 import Text.CSL.Parser ( toRead )
 import Text.CSL.Reference
 import Text.CSL.Style
-import Text.Pandoc.Definition ( Inline (Str,EnDash) )
+import Text.Pandoc.Definition ( Inline (Str) )
 
 evalDate :: Element -> State EvalState [Output]
 evalDate (Date s f fm dl dp dp') = do
@@ -100,20 +100,20 @@ formatDate em k tm dp date
       addZero n = if length n == 1 then '0' : n else n
       addZeros  = reverse . take 5 . flip (++) (repeat '0') . reverse
       formatDatePart False (RefDate y m e d _ _) (DatePart n f _ fm)
-          | "year"  <- n, y /= [] = return $ OYear (formatYear  f y) k fm
-          | "month" <- n, m /= [] = output fm      (formatMonth f m)
-          | "day"   <- n, d /= [] = output fm      (formatDay   f d)
+          | "year"  <- n, y /= [] = return $ OYear (formatYear  f    y) k fm
+          | "month" <- n, m /= [] = output fm      (formatMonth f fm m)
+          | "day"   <- n, d /= [] = output fm      (formatDay   f    d)
           | "month" <- n, m == []
                         , e /= [] = output fm $ term f ("season-0" ++ e)
 
       formatDatePart True (RefDate y m e d _ _) (DatePart n f rd fm)
           | "year"  <- n, y /= [] = OYear (formatYear  f y) k (fm {suffix = []}) : formatDelim
-          | "month" <- n, m /= [] = output (fm {suffix = []}) (formatMonth f m) ++ formatDelim
-          | "day"   <- n, d /= [] = output (fm {suffix = []}) (formatDay   f d) ++ formatDelim
+          | "month" <- n, m /= [] = output (fm {suffix = []}) (formatMonth f fm m) ++ formatDelim
+          | "day"   <- n, d /= [] = output (fm {suffix = []}) (formatDay   f    d) ++ formatDelim
           | "month" <- n, m == []
                         , e /= [] = output (fm {suffix = []}) (term f $ "season-0" ++ e) ++ formatDelim
           where
-            formatDelim = if rd == "-" then [OPan [EnDash]] else [OPan [Str rd]]
+            formatDelim = if rd == "-" then [OPan [Str "\x2013"]] else [OPan [Str rd]]
 
       formatDatePart _ (RefDate _ _ _ _ o _) (DatePart n _ _ fm)
           | "year"  <- n, o /= [] = output fm o
@@ -131,12 +131,13 @@ formatDate em k tm dp date
           | otherwise    = y
           where
             iy = readNum y
-      formatMonth f m
-          | "short"   <- f = getMonth fst
+      formatMonth f fm m
+          | "short"   <- f = getMonth $ period . fst
           | "long"    <- f = getMonth fst
           | "numeric" <- f = m
           | otherwise      = addZero m
           where
+            period     = if stripPeriods fm then filter (/= '.') else id
             getMonth g = maybe m g $ lookup ("month-" ++ addZero m, read $ toRead f) tm
       formatDay f d
           | "numeric-leading-zeros" <- f = addZero d
