@@ -17,7 +17,7 @@ module Text.CSL.Eval.Names where
 
 import Control.Applicative ( (<$>) )
 import Control.Monad.State
-import Data.Char  ( toUpper, isLower, isSpace )
+import Data.Char  ( toUpper, isLower, isUpper, isSpace )
 import Data.List  ( nub )
 import Data.Maybe ( isJust )
 
@@ -69,7 +69,7 @@ evalNames skipEdTrans ns nl d
       agents p s a = concatMapM (formatNames (hasEtAl nl) d p s a) nl
       delim    ops = if d == [] then getOptionVal "names-delimiter" ops else d
       resetEtal    = modify (\s -> s { etal = [] })
-      count  num x = if hasCount nl && num /= []
+      count  num x = if hasCount nl && num /= [] -- FIXME!! le zero!!
                      then [OContrib [] [] [ONum (length num) emptyFormatting] [] []]
                      else x
       hasCount     = or . query hasCount'
@@ -172,19 +172,21 @@ formatNames ea del p s as n
         if isWithLastName o
            then case () of
                   _ | (length as - i) == 1 -> et_al o b t fm d i -- is that correct? FIXME later
-                    | (length as - i) >  1 -> return $ [ODel d, OPan [Ellipses], OSpace] ++ ln
+                    | (length as - i) >  1 -> return $ [ODel d, OPan [Str "\x2026"], OSpace] ++ ln
                     | otherwise            -> return []
            else et_al o b t fm d i
       et_al o b t fm d i
           = when' (gets mode >>= return . not . isSorting) $
             if b || length as <= i
             then return []
-            else case getOptionVal "delimiter-precedes-et-al" o of
-                   "never"  -> return . (++) [OSpace] . output fm =<< getTerm False Long t
-                   "always" -> return . (++) [ODel d] . output fm =<< getTerm False Long t
-                   _        -> if i > 1
-                               then return . (++) [ODel d] . output fm =<< getTerm False Long t
-                               else return . (++) [OSpace] . output fm =<< getTerm False Long t
+            else do x <- getTerm False Long t
+                    when' (return $ x /= []) $
+                          case getOptionVal "delimiter-precedes-et-al" o of
+                            "never"  -> return . (++) [OSpace] $ output fm x
+                            "always" -> return . (++) [ODel d] $ output fm x
+                            _        -> if i > 1
+                                        then return . (++) [ODel d] $ output fm x
+                                        else return . (++) [OSpace] $ output fm x
 
 -- | The first 'Bool' is 'True' if we are evaluating the bibliography.
 -- The 'String' is the cite position. The function also returns the
@@ -223,23 +225,18 @@ isEtAl b os p as
                             else getOptionVal' "et-al-use-first" "et-al-subsequent-min"
                     in if null u then 1 else read u
 
-isPlural :: Plural -> Int -> Bool
-isPlural p l
-    = case p of
-        Always     -> True
-        Never      -> False
-        Contextual -> l > 1
-
 -- | Generate the 'Agent's names applying et-al options, with all
 -- possible permutations to disambiguate colliding citations. The
 -- 'Bool' indicate whether we are formatting the first name or not.
 formatName :: EvalMode -> Bool -> Form -> Formatting -> [Option] -> [NamePart] -> Agent -> [Output]
 formatName m b f fm ops np n
-    | Short     <- f = return $ OName (show n)  shortName       disambdata fm
-    | otherwise      = return $ OName (show n) (longName given) disambdata fm
+    | literal n /= [] = return $ OName (show n)  institution     []         fm
+    | Short      <- f = return $ OName (show n)  shortName       disambdata fm
+    | otherwise       = return $ OName (show n) (longName given) disambdata fm
     where
+      institution = [OStr (literal n) $ form "family"]
       when_ c o = if c /= [] then o else []
-      addAffixes s sf ns = [Output ((OStr s (form sf) { prefix = [], suffix = [] }) : ns) $
+      addAffixes s sf ns = [Output ((oStr' s (form sf) { prefix = [], suffix = [] }) ++ ns) $
                                    emptyFormatting { prefix = prefix (form sf)
                                                    , suffix = suffix (form sf)}]
 
@@ -251,13 +248,17 @@ formatName m b f fm ops np n
       hyphen    = if getOptionVal "initialize-with-hyphen" ops == "false"
                   then getOptionVal "initialize-with" ops
                   else filter (not . isSpace) $ getOptionVal "initialize-with" ops  ++ "-"
-      initial x = if isJust (lookup "initialize-with" ops)
+      isInit  x = length x == 1 && or (map isUpper x)
+      initial x = if isJust (lookup "initialize-with" ops) &&
+                     getOptionVal "initialize" ops /= "false"
                   then if not . and . map isLower $ x
                        then addIn x $ getOptionVal "initialize-with" ops
                        else " " ++ case x of
                                      _:'\'':[] -> x
                                      _         -> x ++ " "
-                  else " " ++ x
+                  else " " ++ if isJust (lookup "initialize-with" ops) && isInit x
+                              then addIn x $ getOptionVal "initialize-with" ops
+                              else x
       addIn x i = if hasHyphen x
                   then head (       takeWhile (/= '-') x) : hyphen ++
                        head (tail $ dropWhile (/= '-') x) : i
@@ -278,26 +279,27 @@ formatName m b f fm ops np n
       givenLong = when_ (givenName  n) . unwords' $ givenName n
       family    = familyName n
 
-      shortName = [OStr (nonDroppingPart n <+> family) (form "family")]
+      shortName = oStr' (nonDroppingPart n <+> family) (form "family")
       longName g = if isSorting m
                    then let firstPart = case getOptionVal "demote-non-dropping-particle" ops of
                                            "never" -> nonDroppingPart n <+> family  <+> droppingPart n
                                            _       -> family  <+> droppingPart n <+> nonDroppingPart n
-                        in [OStr firstPart (form "family")] <++> [OStr g (form "given")] ++ suffCom
+                        in [OStr firstPart (form "family")] <++> oStr' g (form "given") ++ suffCom
                    else if (b && getOptionVal "name-as-sort-order" ops == "first") ||
                            getOptionVal "name-as-sort-order" ops == "all"
                         then let (fam,par) = case getOptionVal "demote-non-dropping-particle" ops of
                                                "never"     -> (nonDroppingPart n <+> family, droppingPart n)
                                                "sort-only" -> (nonDroppingPart n <+> family, droppingPart n)
                                                _           -> (family, droppingPart n <+> nonDroppingPart n)
-                             in OStr fam (form "family") : sortSep g par ++ suffCom
+                             in oStr' fam (form "family") ++ sortSep g par ++ suffCom
                           else oStr' g (form "given") <++>
                                addAffixes (droppingPart n <+> nonDroppingPart n <+> family) "family" suff
 
       disWithGiven = getOptionVal "disambiguate-add-givenname" ops == "true"
-      initialize   = not . null . getOptionVal "initialize-with" $ ops
+      initialize   = isJust . lookup "initialize-with" $ ops
       isLong       = f /= Short && initialize
-      givenRule    = getOptionVal "givenname-disambiguation-rule" ops
+      givenRule    = let gr = getOptionVal "givenname-disambiguation-rule" ops
+                     in if null gr then "by-cite" else gr
       disambdata   = case () of
                        _ | "all-names-with-initials"    <- givenRule
                          , disWithGiven, Short <- f, initialize    -> [longName given]
@@ -323,19 +325,21 @@ formatLabel :: Form -> Formatting -> Bool -> String -> State EvalState [Output]
 formatLabel f fm p s
     | "locator" <- s = when' (gets (citeLocator . cite . env) >>= return . (/=) []) $ do
                        (l,v) <- getLocVar
-                       format l ('-' `elem` v)
-    | "page"    <- s = when' (isVarSet s) $ do
-                       v <- getStringVar s
-                       format  s ('-' `elem` v)
+                       form (\fm' -> return . flip OLoc emptyFormatting . output fm') id l ('-' `elem` v)
+    | "page"    <- s = checkPlural
+    | "volume"  <- s = checkPlural
     | "ibid"    <- s = format' s p
     | otherwise      = format  s p
     where
-      format      = form id
+      checkPlural = when' (isVarSet s) $ do
+                      v <- getStringVar s
+                      format  s ('-' `elem` v)
+      format      = form output id
       format' t b = gets (citePosition . cite . env) >>= \po ->
                     if po == "ibid-with-locator-c" || po == "ibid-c"
-                    then form capital t b
-                    else form id t b
-      form  g t b = return . output fm =<< g . period <$> getTerm (b && p) f t
+                    then form output capital t b
+                    else format t b
+      form o g t b = return . o fm =<< g . period <$> getTerm (b && p) f t
       period      = if stripPeriods fm then filter (/= '.') else id
       capital   x = toUpper (head x) : (tail x)
 
