@@ -23,9 +23,10 @@ import Text.CSL.Pickle
 import Text.CSL.Reference
 import Text.CSL.Input.Json
 import Text.CSL.Input.MODS
+import Text.JSON.Generic
 
 #ifdef USE_BIBUTILS
-import Control.Exception ( bracket )
+import Control.Exception ( bracket, catch )
 import Control.Monad.Trans ( liftIO )
 import System.FilePath ( (</>), (<.>) )
 import System.IO.Error ( isAlreadyExistsError )
@@ -44,6 +45,7 @@ readBiblioFile f
     = case getExt f of
         ".mods"    -> readBiblioFile' f mods_in
         ".bib"     -> readBiblioFile' f biblatex_in
+        ".bibtex"  -> readBiblioFile' f bibtex_in
         ".ris"     -> readBiblioFile' f ris_in
         ".enl"     -> readBiblioFile' f endnote_in
         ".xml"     -> readBiblioFile' f endnotexml_in
@@ -51,20 +53,23 @@ readBiblioFile f
         ".medline" -> readBiblioFile' f medline_in
         ".copac"   -> readBiblioFile' f copac_in
         ".json"    -> readJsonInput f
+        ".native"  -> readFile f >>= return . decodeJSON
         _          -> error $ "citeproc: the format of the bibliographic database could not be recognized\n" ++
                               "using the file extension."
 
 #else
 readBiblioFile f
-    | ".mods" <- getExt f = readModsCollectionFile f
-    | ".json" <- getExt f = readJsonInput f
-    | otherwise           = error $ "citeproc: Bibliography format not supported.\n" ++
-                                    "citeproc-hs was not compiled with bibutils support."
+    | ".mods"   <- getExt f = readModsCollectionFile f
+    | ".json"   <- getExt f = readJsonInput f
+    | ".native" <- getExt f = readFile f >>= return . decodeJSON
+    | otherwise             = error $ "citeproc: Bibliography format not supported.\n" ++
+                                      "citeproc-hs was not compiled with bibutils support."
 #endif
 
 data BibFormat
     = Mods
     | Json
+    | Native
 #ifdef USE_BIBUTILS
     | Bibtex
     | BibLatex
@@ -80,6 +85,7 @@ readBiblioString :: BibFormat -> String -> IO [Reference]
 readBiblioString b s
     | Mods      <- b = return $ readXmlString xpModsCollection s
     | Json      <- b = return $ readJsonInputString s
+    | Native    <- b = return $ decodeJSON s
 #ifdef USE_BIBUTILS
     | Bibtex    <- b = go bibtex_in
     | BibLatex  <- b = go biblatex_in
@@ -128,10 +134,10 @@ createTempDir :: Integer -> FilePath -> IO FilePath
 createTempDir num baseName = do
   sysTempDir <- getTemporaryDirectory
   let dirName = sysTempDir </> baseName <.> show num
-  liftIO $ catch (createDirectory dirName >> return dirName) $
+  liftIO $ Control.Exception.catch (createDirectory dirName >> return dirName) $
       \e -> if isAlreadyExistsError e
-               then createTempDir (num + 1) baseName
-               else ioError e
+            then createTempDir (num + 1) baseName
+            else ioError e
 #endif
 
 getExt :: String -> String
