@@ -16,16 +16,14 @@
 
 module Text.CSL.Input.MODS where
 
-import Text.CSL.Output.Plain ( (<+>) )
-import Text.CSL.Reference
+import Text.CSL.Eval ( split )
+import Text.CSL.Output.Plain ( (<+>), tail' )
 import Text.CSL.Pickle
+import Text.CSL.Reference
 import Text.CSL.Style ( betterThen )
 
 import Data.Char ( isDigit, isLower )
-
-#ifdef USE_HXT
-import Text.XML.HXT.Arrow.Pickle.Xml
-#endif
+import qualified Data.Map as M
 
 -- | Read a file with a single MODS record.
 readModsFile :: FilePath -> IO Reference
@@ -47,11 +45,11 @@ xpReference
                 , (ck,ty,ti,i,d)
                 ,((au,ed,tr),(re,it,pu'),(co,ce))
                 ,((di',pg,vl,is),(nu,sc,ch))
-                , (di,pu,pp)
-                , ((ac,uri),no)
+                , (di,ac,pu,pp)
+                , ((ac',uri),no)
                  ) ->
                ref { refId            = ck `betterThen` take 10 (concat $ words ti)
-                   , refType          = if refType ref /= NoType then refType ref else ty
+                   , refType          = if ty == NoType then refType ref else ty
                    , title            = ti
                    , author           = au
                    , editor           = ed `betterThen` editor           ref
@@ -64,10 +62,10 @@ xpReference
                    , containerAuthor  = containerAuthor  ref
                    , url              = uri
                    , note             = no
-                   , accessed         = ac
                    , isbn             = i
                    , doi              = d
                    , issued           = issued        ref `betterThen` di `betterThen` di'
+                   , accessed         = accessed      ref `betterThen` ac `betterThen` ac'
                    , page             = page          ref `betterThen` pg
                    , volume           = volume        ref `betterThen` vl
                    , issue            = issue         ref `betterThen` is
@@ -85,26 +83,27 @@ xpReference
                       ,(composer  r, collectionEditor r))
                      ,((issued    r, page  r, volume r, issue r)
                       ,(number    r, section   r, chapterNumber r))
-                     , (issued    r, emptyAgents, publisherPlace r)
+                     , (issued    r, accessed r, emptyAgents, publisherPlace r)
                      ,((accessed  r, url r), note r)
                      )) $
       xp6Tuple (xpDefault emptyReference xpRelatedItem)
                (xp5Tuple  xpCiteKey xpRefType xpTitle xpIsbn xpDoi)
-                xpAgents xpPart xpOrigin
-                (xpPair xpUrl xpNote)
+                xpAgents xpPart xpOrigin (xpPair xpUrl xpNote)
 
 xpCiteKey :: PU String
 xpCiteKey
     = xpDefault [] $
-      xpChoice (xpAttr  "ID"         xpText)
-               (xpIElem "identifier" xpText)
+      xpChoice (xpAttr "ID" xpText)
+               (xpElemWithAttrValue "identifier" "type" "citekey" xpText)
                 xpLift
 
-xpOrigin :: PU ([RefDate],[Agent],String)
+xpOrigin :: PU ([RefDate],[RefDate],[Agent],String)
 xpOrigin
-    = xpDefault ([],[],[]) . xpIElem "originInfo" $
-      xpTriple (xpDefault [] $ xpWrap (readDate,show) $
+    = xpDefault ([],[],[],[]) . xpIElem "originInfo" $
+      xp4Tuple (xpDefault [] $ xpWrap (readDate,show) $
                 xpIElem "dateIssued" xpText0)
+               (xpDefault [] $ xpWrap (readDate,show) $
+                xpIElem "dateCaptured" xpText0)
                (xpDefault [] $ xpList $ xpWrap (\s -> Agent [] [] [] s [] [] False, show) $
                 xpIElem "publisher" xpText0)
                (xpDefault [] $ xpIElem "place" $ xpIElem "placeTerm" xpText0)
@@ -112,26 +111,7 @@ xpOrigin
 xpRefType :: PU RefType
 xpRefType
     = xpDefault NoType $
-      xpWrap (readType, const []) xpGenre
-      where
-        readType [] = NoType
-        readType (t:_)
-            | "conference publication" <- t = PaperConference
-            | "periodical"             <- t = ArticleJournal
-            | otherwise                     = Book
-
-xpRefType' :: PU RefType
-xpRefType'
-    = xpDefault NoType $
-      xpWrap (readTypeIn, const []) xpGenre
-      where
-        readTypeIn  [] = NoType
-        readTypeIn t
-            | "book"                   `elem` t = Chapter
-            | "conference publication" `elem` t = PaperConference
-            | "academic journal"       `elem` t = ArticleJournal
-            | "collection"             `elem` t = Chapter
-            | otherwise                         = ArticleJournal
+      xpWrap (readRefType, const []) xpGenre
 
 xpGenre :: PU [String]
 xpGenre
@@ -146,7 +126,7 @@ xpRelatedItem
       xpWrap ( \( (ty,ct)
                 ,((ca,ed,tr),(re,it,pu'),(co,ce))
                 ,((di,pg,vl,is),(nu,sc,ch))
-                , (di',pu,pp)
+                , (di',ac,pu,pp)
                 ) ->
                emptyReference { refType          = ty
                               , containerAuthor  = ca
@@ -160,6 +140,7 @@ xpRelatedItem
                               , composer         = co
                               , collectionEditor = ce
                               , issued           = di `betterThen` di'
+                              , accessed         = ac
                               , page             = pg
                               , volume           = vl
                               , issue            = is
@@ -173,9 +154,9 @@ xpRelatedItem
                       ,(composer        r, collectionEditor r))
                      ,((issued  r, page  r, volume r, issue r)
                       ,(number  r, section   r, chapterNumber r))
-                     , (issued  r, emptyAgents, publisherPlace r)
+                     , (issued  r, accessed  r,emptyAgents, publisherPlace r)
                      )) $
-      xp4Tuple (xpPair  xpRefType' xpTitle)
+      xp4Tuple (xpPair  xpRefType xpTitle)
                 xpAgents xpPart xpOrigin
 
 -- FIXME: join title and subtitle correctly: usare Title per shortTitle.
@@ -314,9 +295,47 @@ xpNote :: PU (String)
 xpNote = xpDefault [] $ xpIElem "note" xpText
 
 readDate :: String -> [RefDate]
-readDate s = if takeWhile isDigit s /= []
-             then return $ RefDate (takeWhile isDigit s) [] [] [] [] []
-             else []
+readDate s = (parseDate         $ takeWhile (/= '/') s) ++
+             (parseDate . tail' $ dropWhile (/= '/') s)
+
+-- | Possible formats: "YYYY", "YYYY-MM", "YYYY-MM-DD".
+parseDate :: String -> [RefDate]
+parseDate s = case split (== '-') (unwords $ words s) of
+                [y,m,d] -> [RefDate y m [] d  [] []]
+                [y,m]   -> [RefDate y m [] [] [] []]
+                [y]     -> if and (map isDigit y)
+                           then [RefDate y  [] [] [] [] []]
+                           else [RefDate [] [] [] [] y  []]
+                _       -> []
 
 emptyAgents :: [Agent]
 emptyAgents  = []
+
+readRefType :: [String] -> RefType
+readRefType [] = NoType
+readRefType (t:_) =
+  case M.lookup t genreTypeMapping of
+    Just x  -> x
+    Nothing -> ArticleJournal       -- Reasonable default (?)
+
+-- The string constants come from http://www.loc.gov/standards/valuelist/marcgt.html, which are used in the
+-- "<genre></genre>" element (http://www.loc.gov/standards/mods/userguide/genre.html)
+genreTypeMapping ::  M.Map String RefType
+genreTypeMapping = M.fromList
+  [ ( "book",                       Book )
+  , ( "periodical",                 ArticleJournal )
+  , ( "newspaper",                  ArticleNewspaper )
+  , ( "encyclopedia",               EntryEncyclopedia )
+  , ( "conference publication",     PaperConference )
+  , ( "academic journal",           ArticleJournal )
+  , ( "collection",                 Chapter )
+  , ( "legal case and case notes",  LegalCase )
+  , ( "legislation",                Legislation )
+  , ( "motion picutre",             MotionPicture )
+  , ( "patent",                     Patent )
+  , ( "review",                     Review )
+  , ( "thesis",                     Thesis )
+  , ( "web page",                   Webpage )
+  , ( "webpage",                    Webpage )
+  , ( "web site",                   Webpage )
+  ]
