@@ -26,6 +26,7 @@ module Text.CSL.Output.Pandoc
     ) where
 
 import Data.Char ( toUpper, toLower, isPunctuation )
+import Data.Maybe ( fromMaybe )
 
 import Text.CSL.Style
 import Text.CSL.Output.Plain
@@ -35,19 +36,22 @@ import Text.Pandoc.Definition
 -- the native 'Pandoc' formats (i.e. immediately readable by pandoc).
 renderPandoc :: Style -> [FormattedOutput] -> [Inline]
 renderPandoc s
-    = proc' (clean s $ isPunctuationInQuote s) . concatMap (flipFlop . render s)
+    = proc (convertQuoted s) . proc' (clean s $ isPunctuationInQuote s) .
+      concatMap (flipFlop . render s)
 
 -- | Same as 'renderPandoc', but the output is wrapped in a pandoc
 -- paragraph block.
 renderPandoc' :: Style -> [FormattedOutput] -> Block
 renderPandoc' s
-    = Para . proc' (clean s $ isPunctuationInQuote s) . concatMap (flipFlop . render s)
+    = Para . proc (convertQuoted s) . proc' (clean s $ isPunctuationInQuote s) .
+      concatMap (flipFlop . render s)
 
 -- | For the testsuite: we use 'Link' and 'Strikeout' to store
 -- "nocase" and "nodecor" rich text formatting classes.
 renderPandoc_ :: Style -> [FormattedOutput] -> [Inline]
 renderPandoc_ s
-    = proc (clean' s $ isPunctuationInQuote s) . concatMap (flipFlop . render s)
+    = proc (convertQuoted s) . proc (clean' s $ isPunctuationInQuote s) .
+      concatMap (flipFlop . render s)
 
 render :: Style -> FormattedOutput -> [Inline]
 render _ (FPan i) = i
@@ -56,14 +60,16 @@ render sty fo
     | FS str fm    <- fo = toPandoc fm $ toStr str
     | FN str fm    <- fo = toPandoc fm $ toStr $ rmZeros str
     | FO     fm xs <- fo = toPandoc fm $ rest xs
+    | FUrl u fm    <- fo = toPandoc fm [Link (toStr u) (u,u)]
     | otherwise = []
     where
       addSuffix f i
           | suffix f /= []
-          , elem (head $ suffix f) ",.:?!"
-          , [head $ suffix f] == lastInline i = i ++ toStr (tail $ suffix f)
-          | suffix f /= []                    = i ++ toStr (       suffix f)
-          | otherwise                         = i
+          , elem (head $ suffix f) ".?!"
+          , lastInline i /= []
+          , last (lastInline i)`elem` ".?!" = i ++ toStr (tail $ suffix f)
+          | suffix f /= []                  = i ++ toStr (       suffix f)
+          | otherwise                       = i
 
       toPandoc f i = addSuffix f $ toStr (prefix f) ++
                      (format f . quote f . proc cleanStrict $ i)
@@ -72,7 +78,6 @@ render sty fo
       quote    f i = if i /= [] && quotes f
                      then [Quoted DoubleQuote . valign f $ i]
                      else valign f i
-
       setCase f i
           | Str     s <- i = Str $ f s
           | otherwise      = i
@@ -121,9 +126,15 @@ toStr :: String -> [Inline]
 toStr = toStr' . entityToChar
     where
       toStr' s
+          |'«':' ':xs <- s = toStr' ("«\8239" ++ xs)
+          |' ':'»':xs <- s = toStr' ("\8239»" ++ xs)
+          |' ':';':xs <- s = toStr' ("\8239;" ++ xs)
+          |' ':':':xs <- s = toStr' ("\8239:" ++ xs)
+          |' ':'!':xs <- s = toStr' ("\8239!" ++ xs)
+          |' ':'?':xs <- s = toStr' ("\8239?" ++ xs)
           |' ':xs <- s = Space   : toStr' xs
           | x :xs <- s = Str [x] : toStr' xs
-          | otherwise   = []
+          | otherwise  = []
 
 cleanStrict :: [Inline] -> [Inline]
 cleanStrict []  = []
@@ -142,8 +153,8 @@ clean s b (i:is)
     | SmallCaps   x <- i = split (isLink  "nodecor" ) (return . SmallCaps  ) x ++ clean s b is
     | Emph        x <- i = split (isLink' "emph"    ) (return . Emph       ) x ++ clean s b is
     | Strong      x <- i = split (isLink' "strong"  ) (return . Strong     ) x ++ clean s b is
-    | Link      x _ <- i = clean' s b (x ++ clean s b is)
-    | otherwise          = clean' s b (i :  clean s b is)
+    | Link      x t <- i = clean' s b (Link x t : clean s b is)
+    | otherwise          = clean' s b (i        : clean s b is)
     where
       unwrap f ls
           | Link x _ : _ <- ls = clean' s b x
@@ -220,6 +231,21 @@ endWithPunct, startWithPunct :: [Inline] -> Bool
 endWithPunct   = and . map (`elem` ".,;:!?") . lastInline
 startWithPunct = and . map (`elem` ".,;:!?") . headInline
 
+convertQuoted :: Style -> [Inline] -> [Inline]
+convertQuoted s = proc convertQuoted'
+    where
+      locale = let l = styleLocale s in case l of [x] -> x; _   -> Locale [] [] [] [] []
+      getQuote  x y = entityToChar . fst . fromMaybe (x,[]) . lookup (y,Long) . localeTermMap $ locale
+      doubleQuotesO = getQuote "\"" "open-quote"
+      doubleQuotesC = getQuote "\"" "close-quote"
+      singleQuotesO = getQuote "'"  "open-inner-quote"
+      singleQuotesC = getQuote "'"  "close-inner-quote"
+      convertQuoted' o
+          | (Quoted DoubleQuote t:xs) <- o = Str doubleQuotesO : t ++ Str doubleQuotesC : convertQuoted' xs
+          | (Quoted SingleQuote t:xs) <- o = Str singleQuotesO : t ++ Str singleQuotesC : convertQuoted' xs
+          | (x                   :xs) <- o = x : convertQuoted' xs
+          | otherwise                      = []
+
 headInline :: [Inline] -> String
 headInline [] = []
 headInline (i:_)
