@@ -257,8 +257,8 @@ instance XmlPickler CSCategory where
 
 xpStyle :: PU Style
 xpStyle
-    = xpWrap ( \ ((v,sc,si,sl,l),(o,m,c,b))   -> Style v sc si sl l o m c b
-             , \ (Style v sc si sl l o m c b) -> ((v,sc,si,sl,l),(o,m,c,b))) $
+    = xpWrap ( \ ((v,sc,si,sl,l),(o,m,c,b))   -> Style v sc si sl l [] o m c b
+             , \ (Style v sc si sl l _ o m c b) -> ((v,sc,si,sl,l),(o,m,c,b))) $
       xpIElem "style" $
       xpPair (xp5Tuple (xpAttrText "version")
                        (xpAttrText "class")
@@ -321,16 +321,18 @@ xpNamesOpt = xpOpt "names-delimiter"
 
 xpNameFormat :: PU [Option]
 xpNameFormat
-    = xpWrap (\(a,b,c,d,e) ->
+    = xpWrap (\(a,b,c,d,e,f) ->
                   catMaybes [ checkOpt "and"                     a
                             , checkOpt "delimiter-precedes-last" b
                             , checkOpt "sort-separator"          c
-                            , checkOpt "initialize-with"         d
-                            , checkOpt "name-as-sort-order"      e
-                            ] , const (Nothing, Nothing, Nothing, Nothing, Nothing)) $
-      xp5Tuple (getOpt "and")
+                            , checkOpt "initialize"              d
+                            , checkOpt "initialize-with"         e
+                            , checkOpt "name-as-sort-order"      f
+                            ] , const (Nothing, Nothing, Nothing, Nothing, Nothing, Nothing)) $
+      xp6Tuple (getOpt "and")
                (getOpt "delimiter-precedes-last")
                (getOpt "sort-separator")
+               (getOpt "initialize")
                (getOpt "initialize-with")
                (getOpt "name-as-sort-order")
 
