@@ -351,7 +351,7 @@ xpAttr name pa
 	    return (Just res, st)
 
 xpElemWithAttrValue :: String -> String -> String -> PU a -> PU a
-xpElemWithAttrValue n a v = xpElem n . xpAddFixedAttr a v
+xpElemWithAttrValue n a v = xpIElem n . xpAddFixedAttr a v
 
 xpAttrFixed	:: String -> String -> PU ()
 xpAttrFixed name val
