@@ -17,10 +17,11 @@
 module Text.CSL.Proc where
 
 import Control.Arrow ( (&&&), (>>>), second )
+import Data.Char ( toLower )
 import Data.List
 import Data.Ord  ( comparing )
 
-import Text.CSL.Eval
+import Text.CSL.Eval hiding ( trim )
 import Text.CSL.Output.Plain
 import Text.CSL.Parser
 import Text.CSL.Proc.Collapse
@@ -81,12 +82,12 @@ citeproc ops s rs cs
 -- according to the 'Style' and assign the citation number to each
 -- 'Reference'.
 procRefs :: Style -> [Reference] -> [Reference]
-procRefs (Style {biblio = mb, csMacros = ms , styleLocale = l, csOptions = opts}) rs
+procRefs (Style {biblio = mb, csMacros = ms , styleLocale = l, styleAbbrevs = as, csOptions = opts}) rs
     = maybe rs process mb
     where
       opts'   b = mergeOptions (bibOptions b) opts
       citNum  x = x { citationNumber = maybe 0 ((+) 1 . fromIntegral) . elemIndex x $ rs }
-      sort_   b = evalSorting (EvalSorting emptyCite {citePosition = "first"})l ms (opts' b) (bibSort b)
+      sort_   b = evalSorting (EvalSorting emptyCite {citePosition = "first"})l ms (opts' b) (bibSort b) as
       process b = sortItems . map (citNum &&& sort_ b) $ rs
 
 sortItems :: Show a => [(a,[Sorting])] -> [a]
@@ -106,12 +107,14 @@ sortItems l
 -- | With a 'Style' and a sorted list of 'Reference's produce the
 -- evaluated output for the bibliography.
 procBiblio :: BibOpts -> Style -> [Reference] -> [[Output]]
-procBiblio bos (Style {biblio = mb, csMacros = ms , styleLocale = l, csOptions = opts}) rs
+procBiblio bos (Style {biblio = mb, csMacros = ms , styleLocale = l,
+                       styleAbbrevs = as, csOptions = opts}) rs
     = maybe [] process mb
     where
       render  b = map (format b) . chkAut [] . filterRefs bos $ rs
       process b = flip map (render b) $ uncurry formatBiblioLayout (layFormat &&& layDelim $ bibLayout b)
-      format  b (p,r) = evalLayout (bibLayout b) (EvalBiblio p) False l ms (mergeOptions (bibOptions b) opts) r
+      format  b (p,r) = evalLayout (bibLayout b) (EvalBiblio p) False l ms
+                                   (mergeOptions (bibOptions b) opts) as r
       chkAut _ []     = []
       chkAut a (x:xs) = if author x `elem` a
                         then ("subsequent",x) : chkAut             a  xs
@@ -143,22 +146,25 @@ filterRefs bos refs
                               | Just v' <- (fromValue x :: Maybe [RefDate]) -> v == [] && v' == [] || v == show v'
                        _                                                    -> False
 
--- | Given the CSL 'Style' and the list of 'Reference's coupled with
--- their position, generate a 'CitationGroup'. The citations are
--- sorted according to the 'Style'.
+-- | Given the CSL 'Style' and the list of 'Cite's coupled with their
+-- 'Reference's, generate a 'CitationGroup'. The citations are sorted
+-- according to the 'Style'.
 procGroup :: Style -> [(Cite, Reference)] -> CitationGroup
-procGroup (Style {citation = ct, csMacros = ms , styleLocale = l, csOptions = opts}) cr
-    = CG authInTxt (layFormat $ citLayout ct) (layDelim $ citLayout ct) result
+procGroup (Style {citation = ct, csMacros = ms , styleLocale = l,
+                  styleAbbrevs = as, csOptions = opts}) cr
+    = CG authIn (layFormat $ citLayout ct) (layDelim $ citLayout ct) (authIn ++ co)
     where
-      authInTxt    = case cr of
+      (co, authIn) = case cr of
                        (c:_) -> if authorInText (fst c)
-                                then foldr (\x _ -> [x]) [] $
-                                     filter ((==) (citeId $ fst c) . citeId . fst) result
-                                else []
-                       _     -> []
+                                then (,) (filter (eqCites (/=) c) $ result
+                                         ) . foldr (\x _ -> [x]) [] .
+                                          filter (eqCites (==) c) $ result
+                                else (,) result []
+                       _     -> (,) result []
+      eqCites eq c = fst >>> citeId &&& citeHash >>> eq (citeId &&& citeHash $ fst c)
       opts'        = mergeOptions (citOptions ct) opts
-      format (c,r) = (,) c $ evalLayout (citLayout ct) (EvalCite c) False l ms opts' r
-      sort_  (c,r) = evalSorting (EvalSorting c) l ms opts' (citSort ct) r
+      format (c,r) = (,) c $ evalLayout (citLayout ct) (EvalCite c) False l ms opts' as r
+      sort_  (c,r) = evalSorting (EvalSorting c) l ms opts' (citSort ct) as r
       process      = map (second (flip Output emptyFormatting) . format &&& sort_)
       result       = sortItems $ process cr
 
@@ -167,35 +173,20 @@ formatBiblioLayout  f d = appendOutput f . addDelim d
 
 formatCitLayout :: Style -> CitationGroup -> [FormattedOutput]
 formatCitLayout s (CG co f d cs)
-    | [a] <- co = formatAuth a : formatCits (setAsSupAu . citeHash . fst $ a) cs
+    | [a] <- co = formatAuth a : formatCits (fst >>> citeId &&& citeHash >>> setAsSupAu $ a) cs
     | otherwise = formatCits id cs
     where
       formatAuth   = formatOutput . localMod
       formatCits g = formatOutputList . appendOutput formatting . addAffixes f .
-                     addDelim d . map (fst &&& localMod >>> uncurry format) . g
-      formatting   = if co /= []
-                     then emptyFormatting
-                     else unsetAffixes f
+                     addDelim d . map (fst &&& localMod >>> uncurry addCiteAffixes) . g
+      formatting   = unsetAffixes f
       localMod     = if cs /= []
                      then uncurry $ localModifiers s (co /= [])
                      else snd
-      setAsSupAu h = map $ \(c,o) -> if citeHash c == h
+      setAsSupAu h = map $ \(c,o) -> if (citeId c, citeHash c) == h
                                      then flip (,) o c { authorInText   = False
                                                        , suppressAuthor = True }
                                      else flip (,) o c
-      format c x = if isNumStyle [x]
-                   then x
-                   else flip Output emptyFormatting $
-                             addCiteAff citePrefix True  c ++ [x] ++
-                             addCiteAff citeSuffix False c
-      addCiteAff g x c =
-          case g c of
-            PlainText  []    -> []
-            PlainText  p | x -> [Output (rtfParser emptyFormatting p) emptyFormatting, OSpace]
-            PlainText  p     -> [OSpace, Output (rtfParser emptyFormatting p) emptyFormatting]
-            PandocText []    -> []
-            PandocText p | x -> [OPan p, OSpace]
-            PandocText p     -> [OPan p]
 
 addAffixes :: Formatting -> [Output] -> [Output]
 addAffixes f os
@@ -238,6 +229,8 @@ localModifiers s b c
           | otherwise        = [True]
       trim [] = []
       trim (o:os)
+          | Output ot f <- o, p <- prefix f,  p /= []
+          , isPunct p         = trim $ Output ot f { prefix = []} : os
           | Output ot f <- o  = if or (query hasOutput ot)
                                 then Output (trim ot) f : os
                                 else Output       ot  f : trim os
@@ -259,29 +252,39 @@ localModifiers s b c
           | OContrib _ "authorsub"
                   _ _ _ <- o = o
           | Output ot f <- o = Output (cleanOutput $ map contribOnly ot) f
+          | OStr    x _ <- o
+          , "ibid" <- filter (/= '.')
+             (map toLower x) = o
           | otherwise        = ONull
+      rmCitNum o
+          | OCitNum {} <- o = ONull
+          | otherwise       = o
       rmContrib [] = []
       rmContrib o
-          | b, isNumStyle o = []
-      rmContrib (o:os)
-          | Output ot f <- o = Output (rmContrib ot) f : rmContrib os
+          | b, isNumStyle o = proc rmCitNum o
+          | otherwise       = rmContrib' o
+      rmContrib' [] = []
+      rmContrib' (o:os)
+          | Output ot f <- o = Output (rmContrib' ot) f : rmContrib' os
+          | ODel _ <- o
+          , OContrib _ "author"
+                     _ _ _ : xs <- os = rmContrib' xs
+          | ODel _ <- o
+          , OContrib _ "authorsub"
+                     _ _ _ : xs <- os = rmContrib' xs
+          | OContrib _ "author" _ _ _ <- o
+          , ODel _ : xs <- os =     rmContrib' xs
+          | OContrib _ "authorsub" _ _ _ <- o
+          , ODel _ : xs <- os =     rmContrib' xs
           | OContrib _ "author"
-                  _ _ _ <- o =        rmContrib os
+                  _ _ _ <- o =     rmContrib' os
           | OContrib _ "authorsub"
-                  _ _ _ <- o =        rmContrib os
-          | otherwise        = o :    rmContrib os
+                  _ _ _ <- o =     rmContrib' os
+          | OStr x _ <- o
+          , "ibid" <- filter (/= '.') (map toLower x) = rmContrib' os
+
+          | otherwise        = o : rmContrib' os
       getRefTerm :: TermMap -> String
       getRefTerm t
           | (("reference", Long), (x,_)) <- t = capitalize x
           | otherwise                         = []
-
-isNumStyle :: [Output] -> Bool
-isNumStyle = null . query authorOrDate
-    where
-      authorOrDate o
-          | OContrib {} <- o = ['a']
-          | OYear    {} <- o = ['a']
-          | OYearSuf {} <- o = ['a']
-          | OStr     {} <- o = ['a']
-          | OPan     {} <- o = ['a']
-          | otherwise        = []
