@@ -18,6 +18,7 @@ module Text.CSL.Style where
 import Data.List ( nubBy, isPrefixOf )
 import Data.Generics ( Typeable, Data, everywhere
                      , everywhere', everything, mkT, mkQ)
+import qualified Data.Map as M
 import Text.JSON
 import Text.Pandoc.Definition ( Inline )
 
@@ -29,6 +30,7 @@ data Style
       , styleInfo          ::  Maybe CSInfo
       , styleDefaultLocale ::  String
       , styleLocale        :: [Locale]
+      , styleAbbrevs       :: [Abbrev]
       , csOptions          :: [Option]
       , csMacros           :: [MacroMap]
       , citation           ::  Citation
@@ -63,6 +65,9 @@ mergeLocales s l ls = doMerge list
       newDate    x = nubBy (\(Date _ a _ _ _ _)
                              (Date _ b _ _ _ _) -> a == b) (concatMap localeDate x ++ localeDate l)
 
+type Abbrev
+    = (String, [(String, M.Map String String)])
+
 type TermMap
     = ((String,Form),(String,String))
 
@@ -222,6 +227,13 @@ data NamePart
     = NamePart String Formatting
       deriving ( Show, Eq, Typeable, Data )
 
+isPlural :: Plural -> Int -> Bool
+isPlural p l
+    = case p of
+        Always     -> True
+        Never      -> False
+        Contextual -> l > 1
+
 isName :: Name -> Bool
 isName x = case x of Name {} -> True; _ -> False
 
@@ -294,12 +306,13 @@ data CSCategory = CSCategory String String String deriving ( Show, Read, Eq, Typ
 -- | The formatted output, produced after post-processing the
 -- evaluated citations.
 data FormattedOutput
-    = FO          Formatting [FormattedOutput]
-    | FN   String Formatting
-    | FS   String Formatting
-    | FDel String
-    | FPan [Inline]
-    | FNull
+    = FO          Formatting [FormattedOutput]          -- ^ List of 'FormatOutput' items
+    | FN   String Formatting                            -- ^ Formatted number
+    | FS   String Formatting                            -- ^ Formatted string
+    | FUrl String Formatting                            -- ^ Formatted uniform resource locator (URL)
+    | FDel String                                       -- ^ Delimeter string
+    | FPan [Inline]                                     -- ^ Pandoc inline elements
+    | FNull                                             -- ^ Null formatting item
       deriving ( Eq, Show )
 
 -- | The 'Output' generated by the evaluation of a style. Must be
@@ -318,6 +331,8 @@ data Output
     | OContrib String    String   [Output] [Output] [[Output]] -- ^ The citation key, the role (author, editor, etc.), the contributor(s),
                                                         -- the output needed for year suf. disambiguation, and everything used for
                                                         -- name disambiguation.
+    | OUrl    String              Formatting            -- ^ A uniform resource locator (URL)
+    | OLoc    [Output]            Formatting            -- ^ The citation's locator
     | Output  [Output]            Formatting            -- ^ Some nested 'Output'
       deriving ( Eq, Ord, Show, Typeable, Data )
 
@@ -357,9 +372,10 @@ emptyAffix = PlainText []
 emptyCite :: Cite
 emptyCite  = Cite [] emptyAffix emptyAffix [] [] [] [] False False False 0
 
--- | A citation group: a list of evaluated citations, the 'Formatting'
--- to be applied to them, and the 'Delimiter' between individual
--- citations.
+-- | A citation group: the first list has a single member when the
+-- citation group starts with an "author-in-text" cite, the
+-- 'Formatting' to be applied, the 'Delimiter' between individual
+-- citations and the list of evaluated citations.
 data CitationGroup = CG [(Cite, Output)] Formatting Delimiter [(Cite, Output)] deriving ( Show, Eq, Typeable, Data )
 
 data BiblioData
@@ -369,11 +385,12 @@ data BiblioData
       } deriving ( Show )
 
 -- | A record with all the data to produce the 'FormattedOutput' of a
--- citation: the citation key, the part of the citation that may be
--- colliding with other citations (the list of contributors for the
--- same year), the data to disambiguate it (all possible contributors
--- and all possible given names), and the disambiguated citation and
--- its year.
+-- citation: the citation key, the part of the formatted citation that
+-- may be colliding with other citations, the form of the citation
+-- when a year suffix is used for disambiguation , the data to
+-- disambiguate it (all possible contributors and all possible given
+-- names), and, after processing, the disambiguated citation and its
+-- year, initially empty.
 data CiteData
     = CD
       { key        ::   String
@@ -381,12 +398,13 @@ data CiteData
       , disambYS   ::  [Output]
       , disambData :: [[Output]]
       , disambed   ::  [Output]
+      , sameAs     ::  [String]
       , citYear    ::   String
       } deriving ( Show, Typeable, Data )
 
 instance Eq CiteData where
-    (==) (CD ka ca _ _ _ _)
-         (CD kb cb _ _ _ _) = ka == kb && ca == cb
+    (==) (CD ka ca _ _ _ _ _)
+         (CD kb cb _ _ _ _ _) = ka == kb && ca == cb
 
 data NameData
     = ND
@@ -421,7 +439,7 @@ formatOutputList = filterUseless . map formatOutput
 formatOutput :: Output -> FormattedOutput
 formatOutput o
     | OSpace             <- o = FDel " "
-    | OPan     i          <- o = FPan i
+    | OPan     i         <- o = FPan i
     | ODel     []        <- o = FNull
     | ODel     s         <- o = FDel s
     | OStr     []      _ <- o = FNull
@@ -430,8 +448,10 @@ formatOutput o
     | OYearSuf s _ _   f <- o = FS s         f
     | ONum     i       f <- o = FS (show  i) f
     | OCitNum  i       f <- o = FN (add00 i) f
+    | OUrl     s       f <- o = FUrl s       f
     | OName  _ s _     f <- o = FO f               (format  s)
     | OContrib _ _ s _ _ <- o = FO emptyFormatting (format  s)
+    | OLoc     os      f <- o = FO f               (format os)
     | Output   os      f <- o = FO f               (format os)
     | otherwise               = FNull
     where
@@ -466,12 +486,6 @@ rmNameHash o
     | OName _ s ss f <- o = OName [] s ss f
     | otherwise           = o
 
--- | Removes all contributors' names.
-rmContribs :: Output -> Output
-rmContribs o
-    | OContrib s r _ _ _ <- o = OContrib s r [] [] []
-    | otherwise               = o
-
 -- | Add, with 'proc', a give name to the family name. Needed for
 -- disambiguation.
 addGivenNames :: [Output] -> [Output]
